#version 330 core 

in vec2 TexCoord;

in vec3 Position_worldspace;
in vec3 Normal_cameraspace;
in vec3 EyeDirection_cameraspace;
in vec3 LightDirection_cameraspace;


out vec4 color;

uniform sampler2D _texture;
//uniform mat4 MV;
uniform mat4 model;
uniform mat4 view;

uniform vec3 mat_diffuse;
//light properties
uniform vec3 LightPosition_worldspace;
uniform vec4 LightColor;
//uniform float LightPower;

void main()
{
	float LightPower = 1.;


	mat4 MV = model * view;

	
	// Material properties
	//vec4 MaterialDiffuseColor = texture(_texture, TexCoord).rgba;
	//vec4 MaterialDiffuseColor = vec4(0.1, 0.75, 0.2, 1.);
	vec4 MaterialDiffuseColor = vec4(mat_diffuse,1.);
	vec4 MaterialAmbientColor = vec4(0.5, 0.5, 0.5, 1.) * MaterialDiffuseColor;
	vec4 MaterialSpecularColor = vec4(0.3,0.3,0.3,1.);



	// Distance to the light
	float distance = length( LightPosition_worldspace - Position_worldspace );

	// Normal of the computed fragment, in camera space
	vec3 n = normalize( Normal_cameraspace );
	// Direction of the light (from the fragment to the light)
	vec3 l = normalize( LightDirection_cameraspace );
	// Cosine of the angle between the normal and the light direction, 
	// clamped above 0
	//  - light is at the vertical of the triangle -> 1
	//  - light is perpendicular to the triangle -> 0
	//  - light is behind the triangle -> 0
	float cosTheta = clamp( dot( n,l ), 0,1 );
	
	// Eye vector (towards the camera)
	vec3 E = normalize(EyeDirection_cameraspace);
	// Direction in which the triangle reflects the light
	vec3 R = reflect(-l,n);
	// Cosine of the angle between the Eye vector and the Reflect vector,
	// clamped to 0
	//  - Looking into the reflection -> 1
	//  - Looking elsewhere -> < 1
	float cosAlpha = clamp( dot( E,R ), 0,1 );
	
	// Ambient : simulates indirect lighting
	color = MaterialAmbientColor;
	
	// Diffuse : "color" of the object
	color = color + MaterialDiffuseColor * LightColor * LightPower * cosTheta;// (distance*distance);
	
	// Specular : reflective highlight, like a mirror
	color = color + MaterialSpecularColor * LightColor * LightPower * pow(cosAlpha,5); // (distance*distance);

}
