#version 330 core
layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 texCoords;


out vec2 TexCoord;


// Output data ; will be interpolated for each fragment.
out vec3 Position_worldspace;
out vec3 Normal_cameraspace;
out vec3 EyeDirection_cameraspace;
out vec3 LightDirection_cameraspace;


uniform mat4 model;
uniform mat4 view;
uniform mat4 projection;

uniform vec3 LightPosition_worldspace;
uniform vec4 LightColor;
//uniform float LightPower;

void main()
{
	float LightPower = 2.;

	//vec3 LightPosition_worldspace = vec3(4.,-10.,4.);
	
	mat4 MVP = projection * view *  model;


	//LightPosition_worldspace = MVP * vec4(LightPosition_worldspace,1.0f).rgb;
	//vec3 LightPosition_normalized = normalize(LightPosition_worldspace);
	vec3 LightPosition_normalized = (LightPosition_worldspace);


    gl_Position = MVP * vec4(position, 1.0f);
    


		
	// Position of the vertex, in worldspace : M * position
	Position_worldspace = (model * vec4(position,1)).xyz;
	

	// Vector that goes from the vertex to the camera, in camera space.
	// In camera space, the camera is at the origin (0,0,0).
	vec3 vertexPosition_cameraspace = ( view * model * vec4(position,1)).xyz;
	EyeDirection_cameraspace = vec3(0,0,0) - vertexPosition_cameraspace;


	// Vector that goes from the vertex to the light, in camera space. M is ommited because it's identity.
	//vec3 LightPosition_cameraspace = ( view * vec4(LightPosition_worldspace,1)).xyz;
	
	vec3 LightPosition_cameraspace = ( view * vec4(LightPosition_normalized ,1)).xyz;

	LightDirection_cameraspace = LightPosition_cameraspace + EyeDirection_cameraspace;

	
	// Normal of the the vertex, in camera space
	Normal_cameraspace = ( view * model * vec4(normal,0)).xyz; // Only correct if ModelMatrix does not scale the model ! Use its inverse transpose if not.
	

	TexCoord = texCoords;
} 

