#version 330 core 

in vec2 TexCoord;

in vec3 Position_worldspace;
in vec3 Normal_cameraspace;
in vec3 EyeDirection_cameraspace;
in vec3 LightDirection_cameraspace;

in vec3 LightDirection_tangentspace;
in vec3 EyeDirection_tangentspace;


in vec3 Tangent_Cameraspace;
in vec3 Bitangent_Cameraspace;
in vec3 VertexNormal_Cameraspace;

in vec3 Tangent_Worldspace;
in vec3 Bitangent_Worldspace;
in vec3 position_Worldspace;
in vec3 normal_Worldspace;


out vec4 color;

uniform sampler2D _textureSampler;

uniform int hasSpec;
uniform sampler2D _specularSampler;

uniform sampler2D _normalSampler;
uniform samplerCube depthMap;

uniform mat4 model;
uniform mat4 view;
uniform mat4 ModelView;

//light properties
uniform vec3 LightPosition_worldspace;
uniform vec4 LightColor;
uniform float LightPower;


uniform mat3 ModelView3;

uniform vec3 viewPos;
uniform float far_plane;

const int samples = 20;

// array of offset direction for sampling
vec3 gridSamplingDisk[20] = vec3[]
(
   vec3(1, 1,  1), vec3( 1, -1,  1), vec3(-1, -1,  1), vec3(-1, 1,  1), 
   vec3(1, 1, -1), vec3( 1, -1, -1), vec3(-1, -1, -1), vec3(-1, 1, -1),
   vec3(1, 1,  0), vec3( 1, -1,  0), vec3(-1, -1,  0), vec3(-1, 1,  0),
   vec3(1, 0,  1), vec3(-1,  0,  1), vec3( 1,  0, -1), vec3(-1, 0, -1),
   vec3(0, 1,  1), vec3( 0, -1,  1), vec3( 0, -1, -1), vec3( 0, 1, -1)
);

float ShadowCalculation(vec3 fragPos, float dummy)
{
    // get vector between fragment position and light position
    vec3 fragToLight = fragPos - LightPosition_worldspace;
    
    // now get current linear depth as the length between the fragment and light position
    float currentDepth = length(fragToLight);
    
    // PCF    
	float shadow = 0.0;
    float bias = 0.10;
    int samples = 20;
    float viewDistance = length(viewPos - fragPos);
    float diskRadius = (1.0 + (viewDistance / far_plane)) / 25.0;
    for(int i = 0; i < samples; ++i)
    {
        float closestDepth = texture(depthMap, fragToLight + gridSamplingDisk[i] * diskRadius).r;
        closestDepth *= far_plane;   // undo mapping [0;1]
        if(currentDepth - bias > closestDepth)
            shadow += 1.0;
    }
    shadow /= float(samples);
  
    return shadow;
}

void main()
{
	//vec3 LightPosition_worldspace = vec3(0.0,1.5,1.0);
	//vec4 LightColor = vec4(1.,1.,1.,1.);
	//float LightPower = 2.;

	// Local normal, in tangent space. 
	//V tex coordinate is inverted because normal map is in TGA (not in DDS) for better quality
	//TODO: see if format of normal map has influence on shading quality
	vec3 TextureNormal_tangentspace = 
		//normalize( texture( _normalSampler, vec2(TexCoord.x,TexCoord.y) ).rgb * 3.0 - 1.0 /*2.0 - 1.0*/);
		normalize( texture( _normalSampler, TexCoord).rgb * 2. - 1.);
	
 
	// Normal of the computed fragment, in camera space
	//vec3 n = normalize( Normal_cameraspace );
	vec3 n = normalize(abs(TextureNormal_tangentspace)) ; 

	//n = normalize( ModelView3 * n);

	//n = normalize(vec3(0.,0.3,-0.3));
	// Direction of the light (from the fragment to the light)
	//vec3 l = normalize( LightDirection_cameraspace );
	vec3 l = normalize(LightDirection_tangentspace);

	// Distance to the light
	float distance = length( LightPosition_worldspace - Position_worldspace );

	// Cosine of the angle between the normal and the light direction, 
	// clamped above 0
	//  - light is at the vertical of the triangle -> 1
	//  - light is perpendicular to the triangle -> 0
	//  - light is behind the triangle -> 0
	float cosTheta = clamp( dot( n,l ), 0,1 );
	
	// Eye vector (towards the camera)
	vec3 E = normalize(EyeDirection_cameraspace);
	// Direction in which the triangle reflects the light
	vec3 R = reflect(-l,n);
	// Cosine of the angle between the Eye vector and the Reflect vector,
	// clamped to 0
	//  - Looking into the reflection -> 1
	//  - Looking elsewhere -> < 1
	float cosAlpha = clamp( dot( E,R ), 0,1 );
	

	//colors calculation
	vec4 MaterialDiffuseColor	= texture(_textureSampler, TexCoord).rgba;
	vec4 MaterialAmbientColor	= vec4(0.5, 0.5, 0.5, 1.)  * MaterialDiffuseColor;
	vec4 MaterialSpecularColor  = vec4(texture(_specularSampler, TexCoord).rgb,1.);
	if(hasSpec == 0)
	{
		MaterialSpecularColor	= vec4(0.3, 0.3, 0.3, 1.);
	}
 
	// Ambient : simulates indirect lighting
	color = MaterialAmbientColor;
	
	// calculate shadow,
	// float bias =   max(0.045 * (1.0 - dot(n, l)), 0.015);
    float shadow =  ShadowCalculation(Position_worldspace , 0);  
	// Diffuse : "color" of the object
	color = color + MaterialDiffuseColor * (1.0-shadow) *LightColor * LightPower * cosTheta; // (distance*distance);
	
	// Specular : reflective highlight, like a mirror
	
	color = color + MaterialSpecularColor *  (1.0-shadow) * LightColor * LightPower * pow(cosAlpha,25); // (distance*distance);
	 

	 
}
