#version 330 core

//default layout, not to be changed in other shaders
//inputs, modelspace
layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 texCoords;

//normal map specific
layout (location = 3) in vec3 tangent;
layout (location = 4) in vec3 bitangent;

out vec2 TexCoord;


// Output data ; will be interpolated for each fragment.
//camera spaces for textures
out vec3 Position_worldspace;
out vec3 Normal_cameraspace;
out vec3 EyeDirection_cameraspace;
out vec3 LightDirection_cameraspace;
out vec3 VertexNormal_Cameraspace;

//camera spaces for normal mapping
out vec3 LightDirection_tangentspace;
out vec3 EyeDirection_tangentspace;


out vec3 Tangent_Cameraspace;
out vec3 Bitangent_Cameraspace;

out vec3 Tangent_Worldspace;
out vec3 Bitangent_Worldspace;
out vec3 position_Worldspace;
out vec3 normal_Worldspace;
//constant uniforms
uniform mat4 model;
uniform mat4 view;
uniform mat4 projection;
uniform mat4 ModelViewProjection;
uniform mat3 ModelView3;


uniform vec3 LightPosition_worldspace;
uniform vec4 LightColor;
uniform float LightPower;

invariant gl_Position;

void main()
{
	//float LightPower = 2.;

    gl_Position = ModelViewProjection * vec4(position.x, position.y, position.z, 1.0f);

	// Normal of the the vertex, in camera space, 2 variants
	//Normal_cameraspace = ( view * model * vec4(normal,0)).xyz; // Only correct if ModelMatrix does not scale the model ! Use its inverse transpose if not.
	Normal_cameraspace 		= ModelView3 * normalize(normal);
    vec3 vertexNormal_camera= (ModelView3 * normalize(normal)				 );
	vec3 tangent_camera		= (ModelView3 * normalize(tangent)				 );
	vec3 bitangent_camera	= (ModelView3 * normalize(bitangent));//(bitangent);
    //vec3 bitangent_camera	= normalize(ModelView3 * cross(normal, tangent));//(bitangent);

	// Position of the vertex, in worldspace : M * position
	Position_worldspace = (model * vec4(position,1)).xyz;
	

	// Vector that goes from the vertex to the camera, in camera space.
	// In camera space, the camera is at the origin (0,0,0).
	vec3 vertexPosition_cameraspace = ( view * model * vec4(position,1)).xyz;
	//vec3 vertexPosition_cameraspace  = ModelView3 * position;
	EyeDirection_cameraspace = vec3(0,0,0) - vertexPosition_cameraspace;
	

	// Vector that goes from the vertex to the light, in camera space. M is ommited because it's identity.
	
	vec3 LightPosition_normalized = (LightPosition_worldspace);
	//vec3 LightPosition_cameraspace = ( view * vec4(LightPosition_worldspace,1)).xyz;
	vec3 LightPosition_cameraspace = ( view * vec4(LightPosition_normalized,1)).xyz;

	//LightDirection_cameraspace = LightPosition_cameraspace + EyeDirection_cameraspace;
	LightDirection_cameraspace = LightPosition_cameraspace;
	LightDirection_cameraspace = LightDirection_cameraspace  + EyeDirection_cameraspace;

	//tangent_camera = normalize(tangent_camera);
	//bitangent_camera = normalize(bitangent_camera);
	//vertexNormal_camera = normalize(vertexNormal_camera);


	//if matrix space is orthogonal, use transpose, as it is faster
	//for now, bitangents seem to be having an issue
	mat3 TBN = transpose
	(
		mat3(
			tangent_camera,
			//bitangent_camera,
			cross(tangent_camera, vertexNormal_camera),
			vertexNormal_camera 	
		)
	); 
	
	// You can use dot products instead of building this matrix and transposing it. See References for details.
	LightDirection_tangentspace = TBN * LightDirection_cameraspace;
	EyeDirection_tangentspace =  TBN * EyeDirection_cameraspace;
	
	LightDirection_cameraspace = LightPosition_cameraspace + EyeDirection_cameraspace;
	
	TexCoord = texCoords;

	//For debugging

	Tangent_Cameraspace = tangent;
	Bitangent_Cameraspace = LightDirection_tangentspace;//cross(normal, tangent);
	VertexNormal_Cameraspace = vertexNormal_camera; 

	position_Worldspace = position;
	normal_Worldspace 	= normal;
	
	Tangent_Worldspace = tangent;
	Bitangent_Worldspace = bitangent;
	
} 

