#version 330 core

// Input vertex data, different for all executions of this shader.
layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 texCoords;

// Output data ; will be interpolated for each fragment.
out vec2 TexCoord;
out vec3 Position_cameraspace;
out vec3 Normal_cameraspace;
out vec4 screenPos;

out vec3 Position_original;
out vec3 Normal_original;


uniform float time;
uniform mat4 ModelViewProjection;
uniform mat4 ModelView;
uniform mat4 InverseView;



const float waveAmp = 0.01;
const float waveLen = 2.0;

const float PI = 3.1415926535897932384626433832795;


float calcOffset(vec3 position, float val1, float val2){

	float x = position.x;
	float z = position.z;

	float radiansX = ((mod(x+z*x*val1, waveLen )/waveLen ) + time * mod(x * 0.8 + z, 1.5)) * 2.0 * PI;
	float radiansZ = ((mod(val2 * (z*x +x*z), waveLen )/waveLen ) + time * 2.0 * mod(x , 2.0) ) * 2.0 * PI;
	
	return waveAmp * 0.5 * (sin(radiansZ) + cos(radiansX));
}


vec3 distortVertex(vec3 vertex){
	float xDistortion = calcOffset(vertex, 0.02, 0.01);
	float yDistortion = calcOffset(vertex, 0.01, 0.03);
	float zDistortion = calcOffset(vertex, 0.015, 0.02);

	return vertex + vec3(xDistortion, yDistortion, zDistortion);
}


void main(){

	//float fact = 0.01;
	//float offset1 =  (cos(vertexPosition_modelspace.x) * sin(time));
	//float offset2 =  (sin(vertexPosition_modelspace.z) * sin(time));
	//
	//float offset = fact * (offset1 * offset2);


	//wave movement animation. This will move the uvs of the normal map to sort of simulate moving waves
	TexCoord = vec2
	(
		texCoords.x + cos(time / 4.) / 250., 
		//clamp(texCoords.y + time * 0.0075,0.,1.)
		abs(texCoords.y + time * 0.00275)
	);

	Position_original = position;
	Normal_original = normal;

	vec3 distortedVertex = distortVertex(position);
	//vec3 distortedNormal = distortVertex(normal);

/*
	Position_cameraspace		= vec3(ModelView * vec4(distortedVertex, 1.0));
  	Normal_cameraspace			= vec3(ModelView * vec4(normal, 0.0));
*/

	screenPos = ModelViewProjection * vec4(distortedVertex, 1);
	gl_Position = screenPos;

	//0.005*vec2( sin(time+1024.0*UV.x),cos(time+768.0*UV.y)) ).xyz
	//UV = (vertexPosition_modelspace.xy+vec2(1,1))/2.0;
}

